*** pregen(<class file name>[,<file path>])
* this pre-processes the class library and creates an intermediate
* text file that is parsed by the help file generator
* this function uses the database abilities of the class library to
* aid in pre-processing
lparameters c_fl_pth, c_fl_nm
local ;
   parm_cnt ;
   nTopicID
   
parm_cnt = parameters()

ot_fl_nm = c_fl_nm + '.rtf'
ot_fl = fcreate(ot_fl_nm)
if ot_fl = -1
   messagebox('File Creation Failed!',16)
   return .F.
endif

* create file name array and add main file as first row
dimension aFiles[1]
aFiles[1] = ot_fl_nm

* create file preamble from help source rtf requirements
CreatePre(ot_fl)

* first record in class file is empty except platform says COMMENT and uniqueid says Class
* each class description ends with a record with COMMENT in platform field and 
* class name in objname field
* step through each valid record and record the following:
* fieldnm		output						output var
* platform									platform specifier
* uniqueid									unique record id
* timestamp									timestamp for modification tracking
* class			Derived from - <contents>	DerFrom
* classloc		Parent Location - <contents>ParLoc
* baseclass		Base Class - <contents>		BasClas
* objname		Class - <contents>			ClasName
* parent		<contents>					SubParent - used in sub object records to specify object they are located in
* properties	<content>					PropList - list of all properties that are set to other than default
* protected		<contents>					ProtList - list of protected things in class
* methods		<contents>					MthdList - list of method code
* objcode									compiled object code
* reserved1									Class - object type specifier - always set to Class
* reserved2		<contents>					ObjCount - count of sub objects in class
* reserved3		<contents>					NuList - list of added things, properties and methods
* reserved4		ToolBar Image - <contents>	TBImage
* reserved5		Insert Image - <contents>	InImage
* reserved6									units object size is specified in
* reserved7		<contents>					ObjDesc - description shown in project window
* reserved8		<contents>					include file name
* user										expansion slot for use by user apps

* output file format
* first line - class library file name
* class record segment
* [ sub object record segment(s) ]
* next class segment, repeat until end of file

* set up input file
if parm_cnt = 2
   in_fl = c_fl_pth + c_fl_nm + '.vcx'
else
   in_fl = c_fl_nm + '.vcx'
endif
use (in_fl)

* set record pointer to top of file and skip header record
go top
skip

* setup variables requiring beginning values
nBaseCount = 1
nTopicID = 1

* creat an array containing each object and its ID
set filter to empty(parent) AND platform <> 'COMMENT'
* count only the records that are visible after filtering
nObjCount = 0
go top
do while !eof()
   nObjCount = nObjCount + 1
   skip
enddo
go top

dimension aClasses[nObjCount,5]
for nObjInx = 1 to nObjCount
   aClasses[nObjInx,1] = baseclass
   aClasses[nObjInx,2] = iif(class=baseclass,' ',class)
   aClasses[nObjInx,3] = objname
   aClasses[nObjInx,4] = nTopicID
   aClasses[nObjInx,5] = recno()
   nTopicID = nTopicID + 1
   skip
endfor
set filter to

* this section repeats for each object in class library
for nObjInx = 1 to nObjCount
   go aClasses[nObjInx,5]
   * get text field values from primary record
   ClsNm = objname
   ClasName = 'Class ' + ClsNm
   * create a jump to derived from class if it is in this library
   nClassMatch = aColScan(@aClasses,3,aClasses[nObjInx,2])
   if nClassMatch > 0
      * create procedure name and jump to code section
      cOutLine = '{\uldb '
      cOutLine = cOutLine + aClasses[nObjInx,2]
      cOutLine = cOutLine + '}{\v IDH_'
      cOutline = cOutLine + alltrim(str(aClasses[nClassMatch,4]))
      cOutLine = cOutLine + '}. '
      DerFrom = 'This class is derived from ' + cOutLine
   else
      DerFrom = 'This class is derived from ' + class
   endif
   ParLoc = 'Parent Location: ' + classloc
   BasClas = 'Base Class: ' + baseclass
   SubParent = ''
   TBImage = 'ToolBar Image: ' + reserved4
   TBImage = strtran(TBImage,'\','|')
   InImage = 'Inserted Image: ' + reserved5
   InImage = strtran(InImage,'\','|')
   ObjDesc = reserved7
   ObjCount = int(val(reserved2))
   
   * create footnote section
   nTID = aClasses[nObjInx,4]
   nTempVal = FootGen(ot_fl,nTID,ClsNm)
   if nTempVal = -1
      messagebox('Source File Footnote Section Creation FAILED.',16)
      return .F.
   endif
   
   * create Topic Title and center it in display
   fputs(ot_fl,'{\f4\fs20 \b ' + ClasName + '\par }')
   
   * set formating back to plain
   fputs(ot_fl,'{\pard\plain \li120 \f4\fs20 ')
   
   * assemble text body segments - seperate paragraphs with '\par'
   * to insert a blank line between each segment requires two '\par' strings
   fputs(ot_fl,DerFrom + '\par')
   fputs(ot_fl,ParLoc + '\par')
   fputs(ot_fl,BasClas + '\par')
   fputs(ot_fl,ObjDesc + '\par \par')
   fputs(ot_fl,TBImage + '\par')
   fputs(ot_fl,InImage + '\par \par')
   
   * check for expanded description in the 'about' method of the class
   AboutDesc(ot_fl)
   
   * add include file data as a topic
   TmpID = IncldProcess(ot_fl,nTopicID,c_fl_pth)
   if TmpID > 0
      * if include processing completed successfully - update nTopicID
      nTopicID = TmpID
   endif
   
   * create properties section
   PropProcess(ot_fl)
   
   * create methods section
   TmpID = MthdProcess(ot_fl,nTopicID)
   if TmpID > 0
      * if methods finished successfully and changed topic id - update nTopicID
      nTopicID = TmpID
   endif
   
   * look at sub objects - at this point implement only listing of them and
   * mentioning which aspects have been set in this main object
   if ObjCount > 1
      * create section header
      fputs(ot_fl,'\par Sub-Object List: \par')
      * step through each sub object and list it
      for o_inx = 2 to ObjCount
         * position record pointer to first/next sub object
         skip
         * get values from sub object record
         sDerFrom = 'Derived from ' + class
         sParLoc = 'Parent Location: ' + classloc
         sBasClas = 'Base Class: ' + baseclass
         sClsNm = objname
         sClasName = 'Class ' + sClsNm
         sSubParent = parent
         nSPropLines = memlines(properties)
         nSProtLines = memlines(protected)
         nSMthdLines = memlines(methods)
         sObjCount = reserved2
         sNuList = reserved3
         sNuLines = alltrim(str(memlines(reserved3)))
         sTBImage = 'ToolBar Image: ' + reserved4
         sInImage = 'Inserted Image: ' + reserved5
         sObjDesc = reserved7
         sChanged = 'Re-Defined: '
         sChanged = sChanged + iif(nSPropLines > 0,'Properties ','')
         sChanged = sChanged + iif(nSMthdLines > 0,'Methods ','')

         * create sub objects listing
         fputs(ot_fl,'Sub-Object = ' + sClsNm + '\par')
         fputs(ot_fl,sChanged + '\par \par')
      endfor
   endif
   * write end of class statement
   fputs(ot_fl,'}')
   * write page break to set up for next topic
   fputs(ot_fl,'{\page }')
   * add in method sections
   if alen(aFiles,1) > 1
      for fInx = 2 to alen(aFiles,1)
         hInFile = fopen(aFiles[FInx])
         if hInFile = -1
            messagebox('Temp File Open FAILED.',16)
         else
            do while NOT feof(hInFile)
               fputs(ot_fl,fgets(hInFile))
            enddo
            fclose(hInFile)
            delete file (aFiles[FInx])
         endif
      endfor
      dimension aFiles[1]
   endif
   * step past footer record
   skip
   skip
endfor

* TOC generation
if nObjCount > 0
   asort(aClasses)
   * create TOC file
   hTOCFile = fcreate(c_fl_nm + '.cnt')
   if hTOCFile = -1
      messagebox('TOC Creation FAILED.',16)
   else
      * create TOC file header
      fputs(hTOCFile,':Base ' + c_fl_nm + '>' + c_fl_nm)
      fputs(hTOCFile,':Title Classes Contained in ' + c_fl_nm + ' Library File')
      fputs(hTOCFile,'1 Classes Contained in ' + c_fl_nm)
      * first order base classes, derived from classes, and classes
      * requires iteritive sorting of array based on BaseClass, Class, and ObjName
      ArrayOrder(@aClasses,1,3)

      * now create TOC based on data in resulting array
      *** step through each cell of the array
      nSC1Row = 1
      do while nSC1Row < alen(aClasses,1)
         nSmC1Row = aSameCount(@aClasses,nSC1Row,1)
         nSC2Row = nSC1Row
         fputs(hTOCFile,'2 Based on Class ' + aClasses[nSC1Row,1])
         do while nSC2Row < nSC1Row + nSmC1Row
            nSmC2Row = aSameCount(@aClasses,nSC2Row,2)
            nSC3Row = nSC2Row
            if len(aClasses[nSC2Row,2]) < 2
               InxPos = '3 '
            else
               InxPos = '4 '
               fputs(hTOCFile,'3 Derived from Class ' + aClasses[nSC2Row,2])
            endif
            do while nSC3Row < nSC2Row + nSmC2Row
               OutLine = InxPos + aClasses[nSC3Row,3] + ;
                         '=IDH_' + alltrim(str(aClasses[nSC3Row,4])) + ;
                         '@' + c_fl_nm + '.hlp>ref'
               fputs(hTOCFile,OutLine)
               nSC3Row = nSC3Row + 1
            enddo
            nSC2Row = nSC2Row + nSmC2Row
         enddo
         nSC1Row = nSC1Row + nSmC1Row
      enddo
      fclose(hTOCFile)
   endif
endif

* write RTF close sequence to file
fputs(ot_fl,'}')
* close output file
fclose(ot_fl)

* close input file and return to main procedure
select (c_fl_nm)
use

return .T.
