*** PropProcess(<output file handle>)
** uses properties field of current database record and produces
** list of changed base properties and a list of user added properties
lparameters hOutFile
local ;
   nArrayDefn, ;
   nFoxProp, ;
   nOrgMemoWidth, ;
   nUserHidn, ;
   nUserProt, ;
   nUserPub

* set memowidth to size large enough to allow each property to occupy one line
nOrgMemoWidth = set('memowidth')
set memowidth to 300

nArrayDefn = 1
nFoxProp = 1
nUserPub = 1
nUserProt = 1
nUserHidn = 1

* process properties section for FoxPro property changes - ignore all user 
* property settings on this pass
PropLines = memlines(properties)
if PropLines > 0
   for PropInx = 1 to PropLines
      cLine = mline(properties,PropInx)
      if NOT empty(cLine)
         cName = substr(cLine,1,atc(' ',cLine,1)-1)
         cDescLine = substr(cLine,atc(' ',cLine,1)+1)
         nUser = atcline(cName,reserved3)
         if nUser = 0
            dimension aFoxProp[nFoxProp,2]
            aFoxProp[nFoxProp,1] = cName
            aFoxProp[nFoxProp,2] = DfltVal(cName)
            nFoxProp = nFoxProp + 1
         endif
      endif
   endfor
endif

* scan Reserved3 for user defined properties, check if hidden or protected, 
* and get default settings. assume default = .F. if not set in properties 
* section
nR3Lines = memlines(reserved3)
if nR3Lines > 0
   for R3Inx = 1 to nR3Lines
      * get first user property line
      cLine = mline(reserved3,R3Inx)
      if NOT empty(cLine)
         * make sure this is not a procedure name
         if left(cLine,1) <> '*'
            * check to ensure there is a description after the property name
            nSpaceChar = atc(' ',cLine,1)
            if nSpaceChar > 0
               * strip the name off the description
               cName = substr(cLine,1,atc(' ',cLine,1)-1)
               * strip description to its own variable
               cPropDesc = alltrim(substr(cLine,atc(' ',cLine,1)+1))
            else
               cName = cLine
               cPropDesc = '??'
            endif
            * check to see if it has an entry in the protected field
            nProtLine = atcline(cName,protected)
            if nProtLine > 0
               * check to see if property has a hidden flag setting
               lHidden = iif(right(mline(protected,nProtLine),1) = '^',.T.,.F.)
            endif
            * check to see if this is an array definition
            lArrayDefn = iif(left(cLine,1)='^',.T.,.F.)
            do case
               case nProtLine = 0 AND NOT lArrayDefn
                  * public property
                  dimension aUserPub[nUserPub,3]
                  aUserPub[nUserPub,1] = cName
                  aUserPub[nUserPub,2] = DfltVal(cName)
                  aUserPub[nUserPub,3] = cPropDesc
                  nUserPub = nUserPub + 1

               case nProtLine > 0 AND NOT lHidden
                  * protected property
                  dimension aUserProt[nUserProt,3]
                  aUserProt[nUserProt,1] = cName
                  aUserProt[nUserProt,2] = DfltVal(cName)
                  aUserProt[nUserProt,3] = cPropDesc
                  nUserProt = nUserProt + 1

               case nProtLine > 0 AND lHidden
                  * hidden property
                  dimension aUserHidn[nUserHidn,3]
                  aUserHidn[nUserHidn,1] = cName
                  aUserHidn[nUserHidn,2] = DfltVal(cName)
                  aUserHidn[nUserHidn,3] = cPropDesc
                  nUserHidn = nUserHidn + 1

               case lArrayDefn
                  * array definition
                  dimension aArrayDefn[nArrayDefn,3]
                  aArrayDefn[nArrayDefn,1] = substr(cName,2)
                  aArrayDefn[nArrayDefn,2] = '.F.'
                  aArrayDefn[nArrayDefn,3] = cPropDesc
                  nArrayDefn = nArrayDefn + 1

            endcase
         endif
      endif
   endfor
endif

* output formatting section
* do FoxPro internal property changes first
if nFoxProp > 1
   fputs(hOutFile,'\par Changes to FoxPro internally defined properties. \par')
   asort(aFoxProp)
   for a_inx = 1 to alen(aFoxProp,1)
      fputs(hOutFile,aFoxProp[a_inx,1] + ':(' + aFoxProp[a_inx,2] + ') \par')
   endfor
endif

* next do public user properties
if nUserPub > 1
   fputs(hOutFile,'\par Public User defined properties. \par')
   asort(aUserPub)
   for a_inx = 1 to alen(aUserPub,1)
      fputs(hOutFile,aUserPub[a_inx,1] + ':(' + aUserPub[a_inx,2] + ') - ' + aUserPub[a_inx,3] + ' \par')
   endfor
endif

* next do protected user properties
if nUserProt > 1
   fputs(hOutFile,'\par Protected User defined properties. \par')
   asort(aUserProt)
   for a_inx = 1 to alen(aUserProt,1)
      fputs(hOutFile,aUserProt[a_inx,1] + ':(' + aUserProt[a_inx,2] + ') - ' + aUserProt[a_inx,3] + ' \par')
   endfor
endif

* next do hidden user properties
if nUserHidn > 1
   fputs(hOutFile,'\par Hidden User defined properties. \par')
   asort(aUserHidn)
   for a_inx = 1 to alen(aUserHidn,1)
      fputs(hOutFile,aUserHidn[a_inx,1] + ':(' + aUserHidn[a_inx,2] + ') - ' + aUserHidn[a_inx,3] + ' \par')
   endfor
endif

* finally list user defined arrays
if nArrayDefn > 1
   fputs(hOutFile,'\par User defined arrays. Arrays are initiallized with all elements set to .F. \par')
   asort(aArrayDefn)
   for a_inx = 1 to alen(aArrayDefn,1)
      fputs(hOutFile,aArrayDefn[a_inx,1] + ':(' + aArrayDefn[a_inx,2] + ') - ' + aArrayDefn[a_inx,3] + ' \par')
   endfor
endif

* set memowidth to starting value
set memowidth to nOrgMemoWidth

return .T.
